@@ -54,12 +54,19 @@
 #include "util.h"
 #include "a2functional.h"
 #include "Option.h"
-#include "StringFormat.h"
+#include "fmt.h"
 #include "RarestPieceSelector.h"
+#include "DefaultStreamPieceSelector.h"
+#include "InorderStreamPieceSelector.h"
+#include "GeomStreamPieceSelector.h"
 #include "array_fun.h"
 #include "PieceStatMan.h"
 #include "wallclock.h"
 #include "bitfield.h"
+#include "SingletonHolder.h"
+#include "Notifier.h"
+#include "WrDiskCache.h"
+#include "RequestGroup.h"
 #ifdef ENABLE_BITTORRENT
 # include "bittorrent_helper.h"
 #endif // ENABLE_BITTORRENT
@@ -67,54 +74,74 @@
 namespace aria2 {
 
 DefaultPieceStorage::DefaultPieceStorage
-(const SharedHandle<DownloadContext>& downloadContext, const Option* option):
-  downloadContext_(downloadContext),
-  bitfieldMan_(new BitfieldMan(downloadContext->getPieceLength(),
-                               downloadContext->getTotalLength())),
-  diskWriterFactory_(new DefaultDiskWriterFactory()),
-  endGame_(false),
-  endGamePieceNum_(END_GAME_PIECE_NUM),
-  logger_(LogFactory::getInstance()),
-  option_(option),
-  pieceStatMan_(new PieceStatMan(downloadContext->getNumPieces(), true)),
-  pieceSelector_(new RarestPieceSelector(pieceStatMan_))
-{}
+(const std::shared_ptr<DownloadContext>& downloadContext, const Option* option)
+ : downloadContext_(downloadContext),
+   bitfieldMan_(new BitfieldMan(downloadContext->getPieceLength(),
+                                downloadContext->getTotalLength())),
+   diskWriterFactory_(new DefaultDiskWriterFactory()),
+   endGame_(false),
+   endGamePieceNum_(END_GAME_PIECE_NUM),
+   option_(option),
+   pieceStatMan_(new PieceStatMan(downloadContext->getNumPieces(), true)),
+   pieceSelector_(make_unique<RarestPieceSelector>(pieceStatMan_)),
+   wrDiskCache_(nullptr)
+{
+  const std::string& pieceSelectorOpt =
+    option_->get(PREF_STREAM_PIECE_SELECTOR);
+  if(pieceSelectorOpt.empty() || pieceSelectorOpt == A2_V_DEFAULT) {
+    streamPieceSelector_ = make_unique<DefaultStreamPieceSelector>
+      (bitfieldMan_);
+  } else if(pieceSelectorOpt == V_INORDER) {
+    streamPieceSelector_ = make_unique<InorderStreamPieceSelector>
+      (bitfieldMan_);
+  } else if(pieceSelectorOpt == A2_V_GEOM) {
+    streamPieceSelector_ = make_unique<GeomStreamPieceSelector>
+      (bitfieldMan_, 1.5);
+  }
+}
 
-DefaultPieceStorage::~DefaultPieceStorage() {
+DefaultPieceStorage::~DefaultPieceStorage()
+{
   delete bitfieldMan_;
 }
 
-SharedHandle<Piece> DefaultPieceStorage::checkOutPiece(size_t index)
+std::shared_ptr<Piece> DefaultPieceStorage::checkOutPiece
+(size_t index, cuid_t cuid)
 {
   bitfieldMan_->setUseBit(index);
 
-  SharedHandle<Piece> piece = findUsedPiece(index);
-  if(piece.isNull()) {
+  std::shared_ptr<Piece> piece = findUsedPiece(index);
+  if(!piece) {
     piece.reset(new Piece(index, bitfieldMan_->getBlockLength(index)));
-
 #ifdef ENABLE_MESSAGE_DIGEST
 
-    piece->setHashAlgo(downloadContext_->getPieceHashAlgo());
+    piece->setHashType(downloadContext_->getPieceHashType());
 
 #endif // ENABLE_MESSAGE_DIGEST
 
     addUsedPiece(piece);
-    return piece;
-  } else {
-    return piece;
   }
+  piece->addUser(cuid);
+  RequestGroup* group = downloadContext_->getOwnerRequestGroup();
+  if((!group || !group->inMemoryDownload()) &&
+     wrDiskCache_ && !piece->getWrDiskCacheEntry()) {
+    // So, we rely on the fact that diskAdaptor_ is not reinitialized
+    // in the session.
+    piece->initWrCache(wrDiskCache_, diskAdaptor_);
+  }
+  return piece;
 }
 
 /**
  * Newly instantiated piece is not added to usedPieces.
  * Because it is waste of memory and there is no chance to use them later.
  */
-SharedHandle<Piece> DefaultPieceStorage::getPiece(size_t index)
+std::shared_ptr<Piece> DefaultPieceStorage::getPiece(size_t index)
 {
-  SharedHandle<Piece> piece;
-  if(0 <= index && index <= bitfieldMan_->getMaxIndex()) {
+  std::shared_ptr<Piece> piece;
+  if(index <= bitfieldMan_->getMaxIndex()) {
     piece = findUsedPiece(index);
-    if(piece.isNull()) {
+    if(!piece) {
       piece.reset(new Piece(index, bitfieldMan_->getBlockLength(index)));
       if(hasPiece(index)) {
         piece->setAllBlock();
@@ -124,53 +151,49 @@ SharedHandle<Piece> DefaultPieceStorage::getPiece(size_t index)
   return piece;
 }
 
-void DefaultPieceStorage::addUsedPiece(const SharedHandle<Piece>& piece)
+void DefaultPieceStorage::addUsedPiece(const std::shared_ptr<Piece>& piece)
 {
-  std::deque<SharedHandle<Piece> >::iterator i =
-    std::lower_bound(usedPieces_.begin(), usedPieces_.end(), piece);
-  usedPieces_.insert(i, piece);
-  if(logger_->debug()) {
-    logger_->debug("usedPieces_.size()=%lu",
-                   static_cast<unsigned long>(usedPieces_.size()));
-  }
+  usedPieces_.insert(piece);
+  A2_LOG_DEBUG(fmt("usedPieces_.size()=%lu",
+                   static_cast<unsigned long>(usedPieces_.size())));
 }
 
-SharedHandle<Piece> DefaultPieceStorage::findUsedPiece(size_t index) const
+std::shared_ptr<Piece> DefaultPieceStorage::findUsedPiece(size_t index) const
 {
-  SharedHandle<Piece> p(new Piece());
+  std::shared_ptr<Piece> p(new Piece());
   p->setIndex(index);
 
-  std::deque<SharedHandle<Piece> >::const_iterator i =
-    std::lower_bound(usedPieces_.begin(), usedPieces_.end(), p);
-  if(i != usedPieces_.end() && (*i) == p) {
-    return *i;
-  } else {
-    p.reset(0);
+  auto i = usedPieces_.find(p);
+  if(i == usedPieces_.end()) {
+    p.reset();
     return p;
+  } else {
+    return *i;
   }
 }
 
 #ifdef ENABLE_BITTORRENT
 
-bool DefaultPieceStorage::hasMissingPiece(const SharedHandle<Peer>& peer)
+bool DefaultPieceStorage::hasMissingPiece(const std::shared_ptr<Peer>& peer)
 {
   return bitfieldMan_->hasMissingPiece(peer->getBitfield(),
                                        peer->getBitfieldLength());
 }
 
 void DefaultPieceStorage::getMissingPiece
-(std::vector<SharedHandle<Piece> >& pieces,
+(std::vector<std::shared_ptr<Piece> >& pieces,
  size_t minMissingBlocks,
  const unsigned char* bitfield,
- size_t length)
+ size_t length,
+ cuid_t cuid)
 {
   const size_t mislen = bitfieldMan_->getBitfieldLength();
-  array_ptr<unsigned char> misbitfield(new unsigned char[mislen]);
+  auto misbitfield = make_unique<unsigned char[]>(mislen);
   size_t blocks = bitfieldMan_->countBlock();
   size_t misBlock = 0;
   if(isEndGame()) {
     bool r = bitfieldMan_->getAllMissingIndexes
-      (misbitfield, mislen, bitfield, length);
+      (misbitfield.get(), mislen, bitfield, length);
     if(!r) {
       return;
     }
@@ -183,20 +206,26 @@ void DefaultPieceStorage::getMissingPiece
     std::random_shuffle(indexes.begin(), indexes.end());
     for(std::vector<size_t>::const_iterator i = indexes.begin(),
           eoi = indexes.end(); i != eoi && misBlock < minMissingBlocks; ++i) {
-      pieces.push_back(checkOutPiece(*i));
-      misBlock += pieces.back()->countMissingBlock();
+      std::shared_ptr<Piece> piece = checkOutPiece(*i, cuid);
+      if(piece->getUsedBySegment()) {
+        // We don't share piece downloaded via HTTP/FTP
+        piece->removeUser(cuid);
+      } else {
+        pieces.push_back(piece);
+        misBlock += piece->countMissingBlock();
+      }
     }
   } else {
     bool r = bitfieldMan_->getAllMissingUnusedIndexes
-      (misbitfield, mislen, bitfield, length);
+      (misbitfield.get(), mislen, bitfield, length);
     if(!r) {
       return;
     }
     while(misBlock < minMissingBlocks) {
       size_t index;
-      if(pieceSelector_->select(index, misbitfield, blocks)) {
-        pieces.push_back(checkOutPiece(index));
-        bitfield::flipBit(misbitfield, blocks, index);
+      if(pieceSelector_->select(index, misbitfield.get(), blocks)) {
+        pieces.push_back(checkOutPiece(index, cuid));
+        bitfield::flipBit(misbitfield.get(), blocks, index);
         misBlock += pieces.back()->countMissingBlock();
       } else {
         break;
@@ -205,53 +234,60 @@ void DefaultPieceStorage::getMissingPiece
   }
 }
 
-static void unsetExcludedIndexes(BitfieldMan& bitfield,
-                                 const std::vector<size_t>& excludedIndexes)
+namespace {
+void unsetExcludedIndexes(BitfieldMan& bitfield,
+                          const std::vector<size_t>& excludedIndexes)
 {
+  using namespace std::placeholders;
   std::for_each(excludedIndexes.begin(), excludedIndexes.end(),
-                std::bind1st(std::mem_fun(&BitfieldMan::unsetBit), &bitfield));
+                std::bind(&BitfieldMan::unsetBit, &bitfield, _1));
 }
+} // namespace
 
 void DefaultPieceStorage::createFastIndexBitfield
-(BitfieldMan& bitfield, const SharedHandle<Peer>& peer)
+(BitfieldMan& bitfield, const std::shared_ptr<Peer>& peer)
 {
-  for(std::vector<size_t>::const_iterator itr =
-        peer->getPeerAllowedIndexSet().begin(),
-        eoi = peer->getPeerAllowedIndexSet().end(); itr != eoi; ++itr) {
-    if(!bitfieldMan_->isBitSet(*itr) && peer->hasPiece(*itr)) {
-      bitfield.setBit(*itr);
+  const auto& is = peer->getPeerAllowedIndexSet();
+  for(const auto& i: is) {
+    if(!bitfieldMan_->isBitSet(i) && peer->hasPiece(i)) {
+      bitfield.setBit(i);
     }
   }
 }
 
 void DefaultPieceStorage::getMissingPiece
-(std::vector<SharedHandle<Piece> >& pieces,
+(std::vector<std::shared_ptr<Piece> >& pieces,
  size_t minMissingBlocks,
- const SharedHandle<Peer>& peer)
+ const std::shared_ptr<Peer>& peer,
+ cuid_t cuid)
 {
   getMissingPiece(pieces, minMissingBlocks,
-                  peer->getBitfield(), peer->getBitfieldLength());
+                  peer->getBitfield(), peer->getBitfieldLength(),
+                  cuid);
 }
 
 
 void DefaultPieceStorage::getMissingPiece
-(std::vector<SharedHandle<Piece> >& pieces,
+(std::vector<std::shared_ptr<Piece> >& pieces,
  size_t minMissingBlocks,
- const SharedHandle<Peer>& peer,
- const std::vector<size_t>& excludedIndexes)
+ const std::shared_ptr<Peer>& peer,
+ const std::vector<size_t>& excludedIndexes,
+ cuid_t cuid)
 {
   BitfieldMan tempBitfield(bitfieldMan_->getBlockLength(),
                            bitfieldMan_->getTotalLength());
   tempBitfield.setBitfield(peer->getBitfield(), peer->getBitfieldLength());
   unsetExcludedIndexes(tempBitfield, excludedIndexes);
   getMissingPiece(pieces, minMissingBlocks,
-                  tempBitfield.getBitfield(), tempBitfield.getBitfieldLength());
+                  tempBitfield.getBitfield(), tempBitfield.getBitfieldLength(),
+                  cuid);
 }
 
 void DefaultPieceStorage::getMissingFastPiece
-(std::vector<SharedHandle<Piece> >& pieces,
+(std::vector<std::shared_ptr<Piece> >& pieces,
  size_t minMissingBlocks,
- const SharedHandle<Peer>& peer)
+ const std::shared_ptr<Peer>& peer,
+ cuid_t cuid)
 {
   if(peer->isFastExtensionEnabled() && peer->countPeerAllowedIndexSet() > 0) {
     BitfieldMan tempBitfield(bitfieldMan_->getBlockLength(),
@@ -259,15 +295,17 @@ void DefaultPieceStorage::getMissingFastPiece
     createFastIndexBitfield(tempBitfield, peer);
     getMissingPiece(pieces, minMissingBlocks,
                     tempBitfield.getBitfield(),
-                    tempBitfield.getBitfieldLength());
+                    tempBitfield.getBitfieldLength(),
+                    cuid);
   }
 }
 
 void DefaultPieceStorage::getMissingFastPiece
-(std::vector<SharedHandle<Piece> >& pieces,
+(std::vector<std::shared_ptr<Piece> >& pieces,
  size_t minMissingBlocks,
- const SharedHandle<Peer>& peer,
- const std::vector<size_t>& excludedIndexes)
+ const std::shared_ptr<Peer>& peer,
+ const std::vector<size_t>& excludedIndexes,
+ cuid_t cuid)
 {
   if(peer->isFastExtensionEnabled() && peer->countPeerAllowedIndexSet() > 0) {
     BitfieldMan tempBitfield(bitfieldMan_->getBlockLength(),
@@ -276,53 +314,61 @@ void DefaultPieceStorage::getMissingFastPiece
     unsetExcludedIndexes(tempBitfield, excludedIndexes);
     getMissingPiece(pieces, minMissingBlocks,
                     tempBitfield.getBitfield(),
-                    tempBitfield.getBitfieldLength());
+                    tempBitfield.getBitfieldLength(),
+                    cuid);
   }
 }
 
-SharedHandle<Piece>
-DefaultPieceStorage::getMissingPiece(const SharedHandle<Peer>& peer)
+std::shared_ptr<Piece>
+DefaultPieceStorage::getMissingPiece
+(const std::shared_ptr<Peer>& peer,
+ cuid_t cuid)
 {
-  std::vector<SharedHandle<Piece> > pieces;
-  getMissingPiece(pieces, 1, peer);
+  std::vector<std::shared_ptr<Piece> > pieces;
+  getMissingPiece(pieces, 1, peer, cuid);
   if(pieces.empty()) {
-    return SharedHandle<Piece>();
+    return nullptr;
   } else {
     return pieces.front();
   }
 }
 
-SharedHandle<Piece> DefaultPieceStorage::getMissingPiece
-(const SharedHandle<Peer>& peer, const std::vector<size_t>& excludedIndexes)
+std::shared_ptr<Piece> DefaultPieceStorage::getMissingPiece
+(const std::shared_ptr<Peer>& peer,
+ const std::vector<size_t>& excludedIndexes,
+ cuid_t cuid)
 {
-  std::vector<SharedHandle<Piece> > pieces;
-  getMissingPiece(pieces, 1, peer, excludedIndexes);
+  std::vector<std::shared_ptr<Piece> > pieces;
+  getMissingPiece(pieces, 1, peer, excludedIndexes, cuid);
   if(pieces.empty()) {
-    return SharedHandle<Piece>();
+    return nullptr;
   } else {
     return pieces.front();
   }
 }
 
-SharedHandle<Piece> DefaultPieceStorage::getMissingFastPiece
-(const SharedHandle<Peer>& peer)
+std::shared_ptr<Piece> DefaultPieceStorage::getMissingFastPiece
+(const std::shared_ptr<Peer>& peer,
+ cuid_t cuid)
 {
-  std::vector<SharedHandle<Piece> > pieces;
-  getMissingFastPiece(pieces, 1, peer);
+  std::vector<std::shared_ptr<Piece> > pieces;
+  getMissingFastPiece(pieces, 1, peer, cuid);
   if(pieces.empty()) {
-    return SharedHandle<Piece>();
+    return nullptr;
   } else {
     return pieces.front();
   }
 }
 
-SharedHandle<Piece> DefaultPieceStorage::getMissingFastPiece
-(const SharedHandle<Peer>& peer, const std::vector<size_t>& excludedIndexes)
+std::shared_ptr<Piece> DefaultPieceStorage::getMissingFastPiece
+(const std::shared_ptr<Peer>& peer,
+ const std::vector<size_t>& excludedIndexes,
+ cuid_t cuid)
 {
-  std::vector<SharedHandle<Piece> > pieces;
-  getMissingFastPiece(pieces, 1, peer, excludedIndexes);
+  std::vector<std::shared_ptr<Piece> > pieces;
+  getMissingFastPiece(pieces, 1, peer, excludedIndexes, cuid);
   if(pieces.empty()) {
-    return SharedHandle<Piece>();
+    return nullptr;
   } else {
     return pieces.front();
   }
@@ -336,37 +382,39 @@ bool DefaultPieceStorage::hasMissingUnusedPiece()
   return bitfieldMan_->getFirstMissingUnusedIndex(index);
 }
 
-SharedHandle<Piece> DefaultPieceStorage::getSparseMissingUnusedPiece
-(size_t minSplitSize, const unsigned char* ignoreBitfield, size_t length)
+std::shared_ptr<Piece> DefaultPieceStorage::getMissingPiece
+(size_t minSplitSize,
+ const unsigned char* ignoreBitfield,
+ size_t length,
+ cuid_t cuid)
 {
   size_t index;
-  if(bitfieldMan_->getSparseMissingUnusedIndex
+  if(streamPieceSelector_->select
      (index, minSplitSize, ignoreBitfield, length)) {
-    return checkOutPiece(index);
+    return checkOutPiece(index, cuid);
   } else {
-    return SharedHandle<Piece>();
+    return nullptr;
   }
 }
 
-SharedHandle<Piece> DefaultPieceStorage::getMissingPiece(size_t index)
+std::shared_ptr<Piece> DefaultPieceStorage::getMissingPiece
+(size_t index,
+ cuid_t cuid)
 {
   if(hasPiece(index) || isPieceUsed(index)) {
-    return SharedHandle<Piece>();
+    return nullptr;
   } else {
-    return checkOutPiece(index);
+    return checkOutPiece(index, cuid);
   }
 }
 
-void DefaultPieceStorage::deleteUsedPiece(const SharedHandle<Piece>& piece)
+void DefaultPieceStorage::deleteUsedPiece(const std::shared_ptr<Piece>& piece)
 {
-  if(piece.isNull()) {
+  if(!piece) {
     return;
   }
-  std::deque<SharedHandle<Piece> >::iterator i = 
-    std::lower_bound(usedPieces_.begin(), usedPieces_.end(), piece);
-  if(i != usedPieces_.end() && (*i) == piece) {
-    usedPieces_.erase(i);
-  }
+  usedPieces_.erase(piece);
+  piece->releaseWrCache(wrDiskCache_);
 }
 
 // void DefaultPieceStorage::reduceUsedPieces(size_t upperBound)
@@ -389,7 +437,7 @@ void DefaultPieceStorage::deleteUsedPiece(const SharedHandle<Piece>& piece)
 //   size_t deleted = 0;
 //   for(Pieces::iterator itr = usedPieces.begin();
 //       itr != usedPieces.end() && deleted < delNum;) {
-//     SharedHandle<Piece>& piece = *itr;
+//     std::shared_ptr<Piece>& piece = *itr;
 //     if(!bitfieldMan->isUseBitSet(piece->getIndex()) &&
 //        piece->countCompleteBlock() <= piece->countBlock()*(fillRate/100.0)) {
 //       logger->info(MSG_DELETING_USED_PIECE,
@@ -405,9 +453,9 @@ void DefaultPieceStorage::deleteUsedPiece(const SharedHandle<Piece>& piece)
 //   return deleted;
 // }
 
-void DefaultPieceStorage::completePiece(const SharedHandle<Piece>& piece)
+void DefaultPieceStorage::completePiece(const std::shared_ptr<Piece>& piece)
 {
-  if(piece.isNull()) {
+  if(!piece) {
     return;
   }
   deleteUsedPiece(piece);
@@ -423,20 +471,32 @@ void DefaultPieceStorage::completePiece(const SharedHandle<Piece>& piece)
   if(downloadFinished()) {
     downloadContext_->resetDownloadStopTime();
     if(isSelectiveDownloadingMode()) {
-      logger_->notice(MSG_SELECTIVE_DOWNLOAD_COMPLETED);
+      A2_LOG_NOTICE(MSG_SELECTIVE_DOWNLOAD_COMPLETED);
       // following line was commented out in order to stop sending request
       // message after user-specified files were downloaded.
       //finishSelectiveDownloadingMode();
     } else {
-      logger_->info(MSG_DOWNLOAD_COMPLETED);
+      A2_LOG_INFO(MSG_DOWNLOAD_COMPLETED);
     }
 #ifdef ENABLE_BITTORRENT
-    if(downloadContext_->hasAttribute(bittorrent::BITTORRENT)) {
-      SharedHandle<TorrentAttribute> torrentAttrs =
-        bittorrent::getTorrentAttrs(downloadContext_);
-      if(!torrentAttrs->metadata.empty()) {
+    if(downloadContext_->hasAttribute(CTX_ATTR_BT)) {
+      if(!bittorrent::getTorrentAttrs(downloadContext_)->metadata.empty()) {
+#ifdef __MINGW32__
+        // On Windows, if aria2 opens files with GENERIC_WRITE access
+        // right, some programs cannot open them aria2 is seeding. To
+        // avoid this situation, re-open the files with read-only
+        // enabled.
+        A2_LOG_INFO("Closing files and re-open them with read-only mode"
+                    " enabled.");
+        diskAdaptor_->closeFile();
+        diskAdaptor_->enableReadOnly();
+        diskAdaptor_->openFile();
+#endif // __MINGW32__
         util::executeHookByOptName(downloadContext_->getOwnerRequestGroup(),
                                    option_, PREF_ON_BT_DOWNLOAD_COMPLETE);
+        SingletonHolder<Notifier>::instance()->
+          notifyDownloadEvent(EVENT_ON_BT_DOWNLOAD_COMPLETE,
+                              downloadContext_->getOwnerRequestGroup());
       }
     }
 #endif // ENABLE_BITTORRENT
@@ -449,12 +509,16 @@ bool DefaultPieceStorage::isSelectiveDownloadingMode()
 }
 
 // not unittested
-void DefaultPieceStorage::cancelPiece(const SharedHandle<Piece>& piece)
+void DefaultPieceStorage::cancelPiece
+(const std::shared_ptr<Piece>& piece, cuid_t cuid)
 {
-  if(piece.isNull()) {
+  if(!piece) {
     return;
   }
-  bitfieldMan_->unsetUseBit(piece->getIndex());
+  piece->removeUser(cuid);
+  if(!piece->getUsed()) {
+    bitfieldMan_->unsetUseBit(piece->getIndex());
+  }
   if(!isEndGame()) {
     if(piece->getCompletedLength() == 0) {
       deleteUsedPiece(piece);
@@ -472,50 +536,61 @@ bool DefaultPieceStorage::isPieceUsed(size_t index)
   return bitfieldMan_->isUseBitSet(index);
 }
 
-uint64_t DefaultPieceStorage::getTotalLength()
+int64_t DefaultPieceStorage::getTotalLength()
 {
   return bitfieldMan_->getTotalLength();
 }
 
-uint64_t DefaultPieceStorage::getFilteredTotalLength()
+int64_t DefaultPieceStorage::getFilteredTotalLength()
 {
   return bitfieldMan_->getFilteredTotalLength();
 }
 
-uint64_t DefaultPieceStorage::getCompletedLength()
+int64_t DefaultPieceStorage::getCompletedLength()
 {
-  uint64_t completedLength =
+  int64_t completedLength =
     bitfieldMan_->getCompletedLength()+getInFlightPieceCompletedLength();
-  uint64_t totalLength = getTotalLength();
+  int64_t totalLength = getTotalLength();
   if(completedLength > totalLength) {
     completedLength = totalLength;
   }
   return completedLength;
 }
 
-uint64_t DefaultPieceStorage::getFilteredCompletedLength()
+int64_t DefaultPieceStorage::getFilteredCompletedLength()
 {
   return bitfieldMan_->getFilteredCompletedLength()+
-    getInFlightPieceCompletedLength();
+    getInFlightPieceFilteredCompletedLength();
 }
 
-size_t DefaultPieceStorage::getInFlightPieceCompletedLength() const
+int64_t DefaultPieceStorage::getInFlightPieceCompletedLength() const
 {
-  return std::accumulate(usedPieces_.begin(), usedPieces_.end(),
-                         0, adopt2nd(std::plus<size_t>(),
-                                     mem_fun_sh(&Piece::getCompletedLength)));
+  int64_t len = 0;
+  for(auto & elem : usedPieces_) {
+    len += elem->getCompletedLength();
+  }
+  return len;
+}
+
+int64_t DefaultPieceStorage::getInFlightPieceFilteredCompletedLength() const
+{
+  int64_t len = 0;
+  for(auto & elem : usedPieces_) {
+    if(bitfieldMan_->isFilterBitSet(elem->getIndex())) {
+      len += elem->getCompletedLength();
+    }
+  }
+  return len;
 }
 
 // not unittested
 void DefaultPieceStorage::setupFileFilter()
 {
-  const std::vector<SharedHandle<FileEntry> >& fileEntries =
+  const std::vector<std::shared_ptr<FileEntry> >& fileEntries =
     downloadContext_->getFileEntries();
   bool allSelected = true;
-  for(std::vector<SharedHandle<FileEntry> >::const_iterator i =
-        fileEntries.begin(), eoi = fileEntries.end();
-      i != eoi; ++i) {
-    if(!(*i)->isRequested()) {
+  for(auto & e : fileEntries) {
+    if(!e->isRequested()) {
       allSelected = false;
       break;
     }
@@ -523,10 +598,9 @@ void DefaultPieceStorage::setupFileFilter()
   if(allSelected) {
     return;
   }
-  for(std::vector<SharedHandle<FileEntry> >::const_iterator i =
-        fileEntries.begin(), eoi = fileEntries.end(); i != eoi; ++i) {
-    if((*i)->isRequested()) {
-      bitfieldMan_->addFilter((*i)->getOffset(), (*i)->getLength());
+  for(auto & e: fileEntries) {
+    if(e->isRequested()) {
+      bitfieldMan_->addFilter(e->getOffset(), e->getLength());
     }
   }
   bitfieldMan_->enableFilter();
@@ -556,40 +630,28 @@ bool DefaultPieceStorage::allDownloadFinished()
 void DefaultPieceStorage::initStorage()
 {
   if(downloadContext_->getFileEntries().size() == 1) {
-    if(logger_->debug()) {
-      logger_->debug("Instantiating DirectDiskAdaptor");
-    }
-    DirectDiskAdaptorHandle directDiskAdaptor(new DirectDiskAdaptor());
+    A2_LOG_DEBUG("Instantiating DirectDiskAdaptor");
+    auto directDiskAdaptor = make_unique<DirectDiskAdaptor>();
     directDiskAdaptor->setTotalLength(downloadContext_->getTotalLength());
     directDiskAdaptor->setFileEntries
       (downloadContext_->getFileEntries().begin(),
        downloadContext_->getFileEntries().end());
 
-    DiskWriterHandle writer =
-      diskWriterFactory_->newDiskWriter(directDiskAdaptor->getFilePath());
-    if(option_->getAsBool(PREF_ENABLE_DIRECT_IO)) {
-      writer->allowDirectIO();
-    }
-
-    directDiskAdaptor->setDiskWriter(writer);
-    diskAdaptor_ = directDiskAdaptor;
+    directDiskAdaptor->setDiskWriter
+      (diskWriterFactory_->newDiskWriter(directDiskAdaptor->getFilePath()));
+    diskAdaptor_ = std::move(directDiskAdaptor);
   } else {
-    if(logger_->debug()) {
-      logger_->debug("Instantiating MultiDiskAdaptor");
-    }
-    MultiDiskAdaptorHandle multiDiskAdaptor(new MultiDiskAdaptor());
+    A2_LOG_DEBUG("Instantiating MultiDiskAdaptor");
+    auto multiDiskAdaptor = make_unique<MultiDiskAdaptor>();
     multiDiskAdaptor->setFileEntries(downloadContext_->getFileEntries().begin(),
                                      downloadContext_->getFileEntries().end());
-    if(option_->getAsBool(PREF_ENABLE_DIRECT_IO)) {
-      multiDiskAdaptor->allowDirectIO();
-    }
     multiDiskAdaptor->setPieceLength(downloadContext_->getPieceLength());
-    multiDiskAdaptor->setMaxOpenFiles
-      (option_->getAsInt(PREF_BT_MAX_OPEN_FILES));
-    diskAdaptor_ = multiDiskAdaptor;
+    diskAdaptor_ = std::move(multiDiskAdaptor);
   }
   if(option_->get(PREF_FILE_ALLOCATION) == V_FALLOC) {
-    diskAdaptor_->enableFallocate();
+    diskAdaptor_->setFileAllocationMethod(DiskAdaptor::FILE_ALLOC_FALLOC);
+  } else if(option_->get(PREF_FILE_ALLOCATION) == V_TRUNC) {
+    diskAdaptor_->setFileAllocationMethod(DiskAdaptor::FILE_ALLOC_TRUNC);
   }
 }
 
@@ -610,18 +672,40 @@ const unsigned char* DefaultPieceStorage::getBitfield()
   return bitfieldMan_->getBitfield();
 }
 
-DiskAdaptorHandle DefaultPieceStorage::getDiskAdaptor() {
+std::shared_ptr<DiskAdaptor> DefaultPieceStorage::getDiskAdaptor() {
   return diskAdaptor_;
 }
 
-size_t DefaultPieceStorage::getPieceLength(size_t index)
+WrDiskCache* DefaultPieceStorage::getWrDiskCache()
+{
+  return wrDiskCache_;
+}
+
+void DefaultPieceStorage::flushWrDiskCacheEntry()
+{
+  if(!wrDiskCache_) {
+    return;
+  }
+  // UsedPieceSet is sorted by piece index. It means we can flush
+  // cache by non-decreasing offset, which is good to reduce disk seek
+  // unless the file is heavily fragmented.
+  for(auto & piece : usedPieces_) {
+    auto ce = piece->getWrDiskCacheEntry();
+    if(ce) {
+      piece->flushWrCache(wrDiskCache_);
+      piece->releaseWrCache(wrDiskCache_);
+    }
+  }
+}
+
+int32_t DefaultPieceStorage::getPieceLength(size_t index)
 {
   return bitfieldMan_->getBlockLength(index);
 }
 
 void DefaultPieceStorage::advertisePiece(cuid_t cuid, size_t index)
 {
-  HaveEntry entry(cuid, index, global::wallclock);
+  HaveEntry entry(cuid, index, global::wallclock());
   haves_.push_front(entry);
 }
 
@@ -633,9 +717,6 @@ DefaultPieceStorage::getAdvertisedPieceIndexes(std::vector<size_t>& indexes,
   for(std::deque<HaveEntry>::const_iterator itr = haves_.begin(),
         eoi = haves_.end(); itr != eoi; ++itr) {
     const HaveEntry& have = *itr;
-    if(have.getCuid() == myCuid) {
-      continue;
-    }
     if(lastCheckTime > have.getRegisteredTime()) {
       break;
     }
@@ -643,6 +724,7 @@ DefaultPieceStorage::getAdvertisedPieceIndexes(std::vector<size_t>& indexes,
   }
 }
 
+namespace {
 class FindElapsedHave
 {
 private:
@@ -651,22 +733,22 @@ public:
   FindElapsedHave(time_t elapsed):elapsed(elapsed) {}
 
   bool operator()(const HaveEntry& have) {
-    if(have.getRegisteredTime().difference(global::wallclock) >= elapsed) {
+    if(have.getRegisteredTime().difference(global::wallclock()) >= elapsed) {
       return true;
     } else {
       return false;
     }
   }
 };
-  
+} // namespace
+
 void DefaultPieceStorage::removeAdvertisedPiece(time_t elapsed)
 {
-  std::deque<HaveEntry>::iterator itr =
-    std::find_if(haves_.begin(), haves_.end(), FindElapsedHave(elapsed));
+  auto itr = std::find_if(haves_.begin(), haves_.end(),
+                          FindElapsedHave(elapsed));
   if(itr != haves_.end()) {
-    if(logger_->debug()) {
-      logger_->debug(MSG_REMOVED_HAVE_ENTRY, haves_.end()-itr);
-    }
+    A2_LOG_DEBUG(fmt(MSG_REMOVED_HAVE_ENTRY,
+                     static_cast<unsigned long>(haves_.end()-itr)));
     haves_.erase(itr, haves_.end());
   }
 }
@@ -676,7 +758,7 @@ void DefaultPieceStorage::markAllPiecesDone()
   bitfieldMan_->setAllBit();
 }
 
-void DefaultPieceStorage::markPiecesDone(uint64_t length)
+void DefaultPieceStorage::markPiecesDone(int64_t length)
 {
   if(length == bitfieldMan_->getTotalLength()) {
     bitfieldMan_->setAllBit();
@@ -691,16 +773,16 @@ void DefaultPieceStorage::markPiecesDone(uint64_t length)
     }
     size_t r = (length%bitfieldMan_->getBlockLength())/Piece::BLOCK_LENGTH;
     if(r > 0) {
-      SharedHandle<Piece> p
+      std::shared_ptr<Piece> p
         (new Piece(numPiece, bitfieldMan_->getBlockLength(numPiece)));
-      
+
       for(size_t i = 0; i < r; ++i) {
         p->completeBlock(i);
       }
 
 #ifdef ENABLE_MESSAGE_DIGEST
 
-      p->setHashAlgo(downloadContext_->getPieceHashAlgo());
+      p->setHashType(downloadContext_->getPieceHashType());
 
 #endif // ENABLE_MESSAGE_DIGEST
 
@@ -715,10 +797,9 @@ void DefaultPieceStorage::markPieceMissing(size_t index)
 }
 
 void DefaultPieceStorage::addInFlightPiece
-(const std::vector<SharedHandle<Piece> >& pieces)
+(const std::vector<std::shared_ptr<Piece> >& pieces)
 {
-  usedPieces_.insert(usedPieces_.end(), pieces.begin(), pieces.end());
-  std::sort(usedPieces_.begin(), usedPieces_.end());
+  usedPieces_.insert(pieces.begin(), pieces.end());
 }
 
 size_t DefaultPieceStorage::countInFlightPiece()
@@ -727,13 +808,13 @@ size_t DefaultPieceStorage::countInFlightPiece()
 }
 
 void DefaultPieceStorage::getInFlightPieces
-(std::vector<SharedHandle<Piece> >& pieces)
+(std::vector<std::shared_ptr<Piece> >& pieces)
 {
   pieces.insert(pieces.end(), usedPieces_.begin(), usedPieces_.end());
 }
 
 void DefaultPieceStorage::setDiskWriterFactory
-(const DiskWriterFactoryHandle& diskWriterFactory)
+(const std::shared_ptr<DiskWriterFactory>& diskWriterFactory)
 {
   diskWriterFactory_ = diskWriterFactory;
 }
@@ -773,4 +854,20 @@ size_t DefaultPieceStorage::getNextUsedIndex(size_t index)
   return bitfieldMan_->countBlock();
 }
 
+void DefaultPieceStorage::onDownloadIncomplete()
+{
+  streamPieceSelector_->onBitfieldInit();
+}
+
+void DefaultPieceStorage::setPieceSelector
+(std::unique_ptr<PieceSelector> pieceSelector)
+{
+  pieceSelector_ = std::move(pieceSelector);
+}
+
+std::unique_ptr<PieceSelector> DefaultPieceStorage::popPieceSelector()
+{
+  return std::move(pieceSelector_);
+}
+
 } // namespace aria2
