@@ -38,7 +38,6 @@
 #include <iterator>
 #include <algorithm>
 
-#include "SharedHandle.h"
 #include "a2io.h"
 #include "FeatureConfig.h"
 #ifdef ENABLE_MESSAGE_DIGEST
@@ -55,7 +54,7 @@ namespace aria2 {
 
 void showVersion() {
   std::cout << PACKAGE << _(" version ") << PACKAGE_VERSION << "\n"
-            << "Copyright (C) 2006, 2013 Tatsuhiro Tsujikawa" << "\n"
+            << "Copyright (C) 2006, 2014 Tatsuhiro Tsujikawa" << "\n"
             << "\n"
             <<
     _("This program is free software; you can redistribute it and/or modify\n"
@@ -84,7 +83,7 @@ void showVersion() {
 
 void showUsage
 (const std::string& keyword,
- const SharedHandle<OptionParser>& oparser,
+ const std::shared_ptr<OptionParser>& oparser,
  const Console& out) {
   out->printf(_("Usage: aria2c [OPTIONS] [URI | MAGNET | TORRENT_FILE |"
                 " METALINK_FILE]..."));
