@@ -32,9 +32,9 @@ class HttpResponseTest : public CppUnit::TestFixture {
   //CPPUNIT_TEST(testGetContentLength_range);
   CPPUNIT_TEST(testGetEntityLength);
   CPPUNIT_TEST(testGetContentType);
-  CPPUNIT_TEST(testDeterminFilename_without_ContentDisposition);
-  CPPUNIT_TEST(testDeterminFilename_with_ContentDisposition_zero_length);
-  CPPUNIT_TEST(testDeterminFilename_with_ContentDisposition);
+  CPPUNIT_TEST(testDetermineFilename_without_ContentDisposition);
+  CPPUNIT_TEST(testDetermineFilename_with_ContentDisposition_zero_length);
+  CPPUNIT_TEST(testDetermineFilename_with_ContentDisposition);
   CPPUNIT_TEST(testGetRedirectURI_without_Location);
   CPPUNIT_TEST(testGetRedirectURI_with_Location);
   CPPUNIT_TEST(testIsRedirect);
@@ -67,9 +67,9 @@ public:
   void testGetContentLength_contentLength();
   void testGetEntityLength();
   void testGetContentType();
-  void testDeterminFilename_without_ContentDisposition();
-  void testDeterminFilename_with_ContentDisposition_zero_length();
-  void testDeterminFilename_with_ContentDisposition();
+  void testDetermineFilename_without_ContentDisposition();
+  void testDetermineFilename_with_ContentDisposition_zero_length();
+  void testDetermineFilename_with_ContentDisposition();
   void testGetRedirectURI_without_Location();
   void testGetRedirectURI_with_Location();
   void testIsRedirect();
@@ -143,7 +143,7 @@ void HttpResponseTest::testGetContentType()
                        httpResponse.getContentType());
 }
 
-void HttpResponseTest::testDeterminFilename_without_ContentDisposition()
+void HttpResponseTest::testDetermineFilename_without_ContentDisposition()
 {
   HttpResponse httpResponse;
   auto httpRequest = make_unique<HttpRequest>();
@@ -155,10 +155,10 @@ void HttpResponseTest::testDeterminFilename_without_ContentDisposition()
   httpResponse.setHttpRequest(std::move(httpRequest));
 
   CPPUNIT_ASSERT_EQUAL(std::string("aria2-1.0.0.tar.bz2"),
-                       httpResponse.determinFilename());
+                       httpResponse.determineFilename());
 }
 
-void HttpResponseTest::testDeterminFilename_with_ContentDisposition_zero_length
+void HttpResponseTest::testDetermineFilename_with_ContentDisposition_zero_length
 ()
 {
   HttpResponse httpResponse;
@@ -173,10 +173,10 @@ void HttpResponseTest::testDeterminFilename_with_ContentDisposition_zero_length
   httpResponse.setHttpRequest(std::move(httpRequest));
 
   CPPUNIT_ASSERT_EQUAL(std::string("aria2-1.0.0.tar.bz2"),
-                       httpResponse.determinFilename());
+                       httpResponse.determineFilename());
 }
 
-void HttpResponseTest::testDeterminFilename_with_ContentDisposition()
+void HttpResponseTest::testDetermineFilename_with_ContentDisposition()
 {
   HttpResponse httpResponse;
   auto httpHeader = make_unique<HttpHeader>();
@@ -191,7 +191,7 @@ void HttpResponseTest::testDeterminFilename_with_ContentDisposition()
   httpResponse.setHttpRequest(std::move(httpRequest));
 
   CPPUNIT_ASSERT_EQUAL(std::string("aria2-current.tar.bz2"),
-                       httpResponse.determinFilename());
+                       httpResponse.determineFilename());
 }
 
 void HttpResponseTest::testGetRedirectURI_without_Location()
@@ -220,16 +220,49 @@ void HttpResponseTest::testGetRedirectURI_with_Location()
 void HttpResponseTest::testIsRedirect()
 {
   HttpResponse httpResponse;
-  auto httpHeader = make_unique<HttpHeader>();
-  httpHeader->setStatusCode(200);
-  httpHeader->put(HttpHeader::LOCATION,
-                  "http://localhost/download/aria2-1.0.0.tar.bz2");
+  httpResponse.setHttpHeader(make_unique<HttpHeader>());
 
-  httpResponse.setHttpHeader(std::move(httpHeader));
+  httpResponse.getHttpHeader()->setStatusCode(301);
   CPPUNIT_ASSERT(!httpResponse.isRedirect());
 
+  httpResponse.getHttpHeader()->setStatusCode(200);
+  CPPUNIT_ASSERT(!httpResponse.isRedirect());
+
+  httpResponse.getHttpHeader()->put
+    (HttpHeader::LOCATION,
+     "http://localhost/download/aria2-1.0.0.tar.bz2");
+
+  CPPUNIT_ASSERT(!httpResponse.isRedirect());
+
+  httpResponse.getHttpHeader()->setStatusCode(300);
+  CPPUNIT_ASSERT(httpResponse.isRedirect());
+
   httpResponse.getHttpHeader()->setStatusCode(301);
   CPPUNIT_ASSERT(httpResponse.isRedirect());
+
+  httpResponse.getHttpHeader()->setStatusCode(302);
+  CPPUNIT_ASSERT(httpResponse.isRedirect());
+
+  httpResponse.getHttpHeader()->setStatusCode(303);
+  CPPUNIT_ASSERT(httpResponse.isRedirect());
+
+  httpResponse.getHttpHeader()->setStatusCode(304);
+  CPPUNIT_ASSERT(!httpResponse.isRedirect());
+
+  httpResponse.getHttpHeader()->setStatusCode(305);
+  CPPUNIT_ASSERT(!httpResponse.isRedirect());
+
+  httpResponse.getHttpHeader()->setStatusCode(306);
+  CPPUNIT_ASSERT(!httpResponse.isRedirect());
+
+  httpResponse.getHttpHeader()->setStatusCode(307);
+  CPPUNIT_ASSERT(httpResponse.isRedirect());
+
+  httpResponse.getHttpHeader()->setStatusCode(308);
+  CPPUNIT_ASSERT(httpResponse.isRedirect());
+
+  httpResponse.getHttpHeader()->setStatusCode(309);
+  CPPUNIT_ASSERT(!httpResponse.isRedirect());
 }
 
 void HttpResponseTest::testIsTransferEncodingSpecified()
@@ -333,15 +366,18 @@ void HttpResponseTest::testValidateResponse()
     httpResponse.validateResponse();
     CPPUNIT_FAIL("exception must be thrown.");
   } catch(Exception& e) {
+    // success
   }
 
-  httpResponse.getHttpHeader()->put
-    (HttpHeader::LOCATION,
-     "http://localhost/archives/aria2-1.0.0.tar.bz2");
-  try {
+  httpResponse.getHttpHeader()->put(HttpHeader::LOCATION, "http://a/b");
+  httpResponse.validateResponse();
+
+  httpResponse.getHttpHeader()->setStatusCode(201);
+  try{
     httpResponse.validateResponse();
+    CPPUNIT_FAIL("exception must be thrown.");
   } catch(Exception& e) {
-    CPPUNIT_FAIL("exception must not be thrown.");
+    // success
   }
 }
 
@@ -654,14 +690,20 @@ void HttpResponseTest::testGetDigest()
      "MD5=LJDK2+9ClF8Nz/K5WZd/+A==");
   std::vector<Checksum> result;
   httpResponse.getDigest(result);
+#ifdef USE_INTERNAL_MD
+  CPPUNIT_ASSERT_EQUAL((size_t)2, result.size());
+#else
   CPPUNIT_ASSERT_EQUAL((size_t)3, result.size());
+#endif
 
   Checksum c = result[0];
+#ifndef USE_INTERNAL_MD
   CPPUNIT_ASSERT_EQUAL(std::string("sha-256"), c.getHashType());
   CPPUNIT_ASSERT_EQUAL(std::string("f83f271ae773dc6fe4a6454a41e0eb237c43e7bbf451e426cc60993a4d379ec5"),
                        util::toHex(c.getDigest()));
 
   c = result[1];
+#endif
   CPPUNIT_ASSERT_EQUAL(std::string("sha-1"), c.getHashType());
   CPPUNIT_ASSERT_EQUAL(std::string("f36003f22b462ffa184390533c500d8989e9f681"),
                        util::toHex(c.getDigest()));
